#include <qmemarray.h>

#include "cryptdevice.h"

CryptDevice::CryptDevice(QIODevice *underlyingDevice)
    : underlyingDevice(underlyingDevice)
{
    setType(IO_Sequential);
}

bool CryptDevice::open(int mode)
{
    bool underlyingOk;
    if (underlyingDevice->isOpen()) 
        underlyingOk = (underlyingDevice->mode() != mode);
    else 
        underlyingOk = underlyingDevice->open(mode);

    if (underlyingOk) {
        setState(IO_Open);
        return true;
    }
    return false;
}

void CryptDevice::close()
{
    underlyingDevice->close();
    setState(0);
}

void CryptDevice::flush()
{
    underlyingDevice->flush();
}

QIODevice::Offset CryptDevice::size() const
{
    return underlyingDevice->size();
}

QIODevice::Offset CryptDevice::at() const
{
    return 0;
}

bool CryptDevice::at(Offset /* offset */)
{
    return false;
}

Q_LONG CryptDevice::readBlock(char *data, Q_ULONG maxSize)
{
    Q_ULONG ungetchRead = 0;
    while (!ungetchBuffer.isEmpty() && ungetchRead < maxSize)
        data[ungetchRead++] = ungetchBuffer.pop();
 
    Q_LONG deviceRead = underlyingDevice->readBlock(data + ungetchRead, maxSize - ungetchRead);
    if (deviceRead == -1)
        return -1;
   
    for (Q_LONG i = 0; i < deviceRead; ++i)
        data[i] = data[ungetchRead + i] ^ 0x5E;

    return ungetchRead + deviceRead;
}

Q_LONG CryptDevice::writeBlock(const char *data, Q_ULONG size) 
{
    QByteArray buffer(size);
    for (Q_ULONG i = 0; i < size; ++i)
        buffer[i] = data[i] ^ 0x5E;
    return underlyingDevice->writeBlock(buffer.data(), size);
}

int CryptDevice::getch()
{
    char ch;
    if (readBlock(&ch, 1) == 1)
        return (uchar)ch;
    else
        return -1;
}

int CryptDevice::putch(int ch)
{
    char data = (char)ch;
    if (writeBlock(&data, 1) == 1)
        return ch;
    else
        return -1;
}

int CryptDevice::ungetch(int ch)
{
    ungetchBuffer.push((char)ch);
    return ch;
}
